package org.xbib.netty.http.server.endpoint;

import org.xbib.netty.http.common.HttpMethod;
import org.xbib.netty.http.server.api.EndpointDescriptor;
import org.xbib.netty.http.server.api.ServerRequest;

import static io.netty.handler.codec.http.HttpHeaderNames.CONTENT_TYPE;

public class HttpEndpointDescriptor implements EndpointDescriptor, Comparable<HttpEndpointDescriptor> {

    private final String path;

    private final HttpMethod method;

    private final String contentType;

    public HttpEndpointDescriptor(ServerRequest serverRequest) {
        this.path = extractPath(serverRequest.getRequestURI());
        this.method = Enum.valueOf(HttpMethod.class, serverRequest.getMethod().name());
        this.contentType = serverRequest.getHeaders().get(CONTENT_TYPE);
    }

    @Override
    public String getSortKey() {
        return path;
    }

    public String getPath() {
        return path;
    }

    public HttpMethod getMethod() {
        return method;
    }

    public String getContentType() {
        return contentType;
    }

    @Override
    public String toString() {
        return "[HttpEndpointDescriptor:path=" + path + ",method=" + method + ",contentType=" + contentType + "]";
    }

    @Override
    public int hashCode() {
        return toString().hashCode();
    }

    @Override
    public boolean equals(Object o) {
        return o instanceof HttpEndpointDescriptor && toString().equals(o.toString());
    }

    @Override
    public int compareTo(HttpEndpointDescriptor o) {
        return toString().compareTo(o.toString());
    }

    private static String extractPath(String uri) {
        String path = uri;
        int pos = uri.lastIndexOf('#');
        path = pos >= 0 ? path.substring(0, pos) : path;
        pos = uri.lastIndexOf('?');
        path = pos >= 0 ? path.substring(0, pos) : path;
        return path;
    }
}
